import datetime
import logging
import os
import traceback
from dataclasses import dataclass
from typing import Optional

from dotenv import load_dotenv
from elasticsearch import Elasticsearch

load_dotenv(verbose=False)

@dataclass
class ElasticLoggerConfig:
    hosts: str
    api_key: str
    index_name: str
    application_name: str
    cert_path: str = "./logger/elasticsearch-ca.crt"


def format_time(record: logging.LogRecord, datefmt: str = None) -> str:
    """
    Format the time for the log record.
    :param record: The log record.
    :param datefmt: Optional date format string.
    :return: Formatted timestamp string.
    """
    dt = datetime.datetime.fromtimestamp(record.created, datetime.timezone.utc)
    return dt.strftime(datefmt or "%Y-%m-%dT%H:%M:%SZ")


class LogHandler(logging.Handler):
    def __init__(self, config: ElasticLoggerConfig):
        """
        Custom logging handler to send logs to Elasticsearch.
        :param config: Configuration object containing connection details.
        """
        super().__init__()
        self.client = Elasticsearch(
            hosts=[host.strip() for host in config.hosts.split(",")],
            api_key=config.api_key,
            verify_certs=False,
            ca_certs=config.cert_path
        )
        self.index_name = config.index_name
        self.application_name = config.application_name

        # Ensure the index exists
        if not self.client.indices.exists(index=self.index_name):
            self.client.indices.create(
                index=self.index_name,
                body={
                    "settings": {
                        "number_of_shards": 1,
                        "number_of_replicas": 1
                    },
                    "mappings": {
                        "properties": {
                            "timestamp": {"type": "date"},
                            "level": {"type": "keyword"},
                            "message": {"type": "text"},
                            "logger": {"type": "keyword"},
                            "filename": {"type": "keyword"},
                            "line": {"type": "integer"},
                            "function": {"type": "keyword"},
                            "application_name": {"type": "keyword"},
                            "stack_trace": {"type": "text"}
                        }
                    }
                }
            )

    def emit(self, record: logging.LogRecord):
        """
        Emit a log record to Elasticsearch.
        :param record: The log record.
        """
        stack_trace = None
        if record.exc_info:
            stack_trace = ''.join(traceback.format_exception(*record.exc_info))

        log_entry = {
            "timestamp": format_time(record),
            "level": record.levelname,
            "message": record.getMessage(),
            "logger": record.name,
            "filename": record.pathname,
            "line": record.lineno,
            "function": record.funcName,
            "application_name": self.application_name,
            "stack_trace": stack_trace,
        }

        try:
            self.client.index(index=self.index_name, body=log_entry)
        except Exception as e:
            print(f"Failed to send log to Elasticsearch: {e}")


def _init_module_logger() -> logging.Logger:
    """
    Initializes (and returns) a logger at import‐time using environment variables.
    If you prefer, you could let this function take optional parameters or
    a config object. But here we do it all from the env so it's truly "ready to go."
    """
    # Gather config from environment
    hosts = os.getenv("ELASTICSEARCH_HOSTS")
    api_key = os.getenv("ELASTICSEARCH_API_KEY")
    index_name = os.getenv("ELASTICSEARCH_INDEX")
    application_name = os.getenv("APPLICATION_NAME")
    cert_path = os.getenv("ELASTICSEARCH_CERT_PATH", "./logger/elasticsearch-ca.crt")

    if not all([hosts, api_key, index_name, application_name, cert_path]):
        raise ValueError("One or more required environment variables (ELASTICSEARCH_*) are not set.")

    config = ElasticLoggerConfig(
        hosts=hosts,
        api_key=api_key,
        index_name=index_name,
        application_name=application_name,
        cert_path=cert_path
    )

    # Create logger
    logger = logging.getLogger("ElasticLogger")
    logger.setLevel(logging.INFO)

    # Only add handlers if no handlers are attached yet (prevents double‐logging)
    if not logger.handlers:
        es_handler = LogHandler(config)
        logger.addHandler(es_handler)

        console_handler = logging.StreamHandler()
        console_handler.setFormatter(logging.Formatter("%(asctime)s - %(levelname)s - %(message)s"))
        logger.addHandler(console_handler)

    return logger


#: A ready‐to‐use module‐level logger; just import and go.
logger = _init_module_logger()

__all__ = ["logger", "LogHandler", "ElasticLoggerConfig"]
